import wepy from 'wepy'
export default class baseMixin extends wepy.mixin {
  /**
   * [公共方法]
   * @param  {[type]}  item [description]
   * @return {Boolean}      [description]
   */
  noop() {
    return null;
  }
  hasOwn(obj, type) {
    return Object.prototype.hasOwnProperty.call(obj, type);
  }

  /**
   * [isXXX 基础方法]
   * @param  {[type]}  item [description]
   * @return {Boolean}      [description]
   */
  isUndefined(item) {
    return typeof item === 'undefined';
  }
  isDefined(item) {
    return !this.isUndefined(item);
  }
  isString(item) {
    return typeof item === 'string';
  }
  isNumber(item) {
    return typeof item === 'number';
  }
  isArray(item) {
    return Object.prototype.toString.apply(item) === '[object Array]';
  }
  isObject(item) {
    return typeof item === 'object' && !this.isArray(item);
  }
  isFunction(item) {
    return typeof item === 'function';
  }

  /**
   * [getXXX 增强方法]
   * @param  {[type]}  item [description]
   * @return {Boolean}      [description]
   */
  getString(item, defaultStr) {
    if (this.isString(item)) return item.trim();
    if (this.isNumber(item)) return `${item}`.trim();
    return defaultStr || '';
  }
  getNumber(item, defaultNum) {
    var matches = this.getString(item).match(/\d+/);
    return this.isNumber(matches && +matches[0]) ? +matches[0] : defaultNum;
  }
  getArray(item, defaultArr) {
    return this.isArray(item) ? item : (defaultArr || []);
  }
  getObject(item, defaultObj) {
    return this.isObject(item) ? item : (defaultObj || {});
  }
  getFunction(item) {
    return this.isFunction(item) ? item : noop;
  }

  /**
   * [JSON方法]
   * @param  {[type]}  item [description]
   * @return {Boolean}      [description]
   */
  $json(item) {
    let str = {type: Object.prototype.toString.call(item)}
    try {
      str = JSON.stringify(item)
    } catch (e) {
      str.error = e && e.stack || ''
    }
    return this.isString(str) ? str : this.$json(str)
  }
  $parse(item) {
    let obj = {type: Object.prototype.toString.call(item)}
    try {
      obj = JSON.parse(item)
    } catch (e) {
      obj.error = e && e.stack || ''
    }
    return this.isObject(obj) ? obj : this.$parse(obj)
  }

  /**
   * [功能方法]
   * @param  {[type]}  item [description]
   * @return {Boolean}      [description]
   */
  isPhone(str) {
    return /^1\d{10}$/.test(str)
  }
  $alert(item = '标题', item2) {
    const param = this.isObject(item) ? Object.assign({
      // 首参数为obj
      title: 'title', content: 'content'
    }, item) : this.isString(item) ? this.isString(item2) ? {
      // 俩参数均为字符串
      title: item, content: item2
    } : {
      // 只有首参为字符串
      title: '', content: item
    } : {
      // 尝试转换字符串
      title: item.toString ? item.toString() : '参数异常'
    }
    wx.showModal(Object.assign({
      showCancel: false
    }, param))
  }
  $info(str) {
    console.log('INFO:: %s', str)
  }
  $error(str) {
    console.log('ERROR:: %s', str)
  }
  $trimStart(str, char) {
    let r2 = new RegExp('^' + char + '+')
    return str.replace(r2, '')
  }
  $remove(array, func) {
    return array.filter(func)
  }
  generateShareID () {
    let d = new Date().getTime()
    if (typeof performance !== 'undefined' && typeof performance.now === 'function') {
      d += performance.now()
    }
    return 'xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
      let r = (d + Math.random() * 16) % 16 | 0
      d = Math.floor(d / 16)
      return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16)
    })
  }
  $formatMoney(money) {
    return (money/100).toFixed(2)
  }
  $randomNum(minNum,maxNum){
    return parseInt(Math.random()*(maxNum-minNum+1)+minNum,10);
  }
  $getRandomColor() {
    let rgb = []
    for (let i = 0; i < 3; ++i) {
      let color = Math.floor(Math.random() * 256).toString(16)
      color = color.length == 1 ? '0' + color : color
      rgb.push(color)
    }
    return '#' + rgb.join('')
  }
  $getShareCfg() {
    let shareArr = wepy.getStorageSync('share_cfg');
    if (shareArr.length > 0) {
      let i = this.$randomNum(0, shareArr.length - 1);
      return shareArr[i];
    } else {
      return {
        str: '怀旧经典游戏',
        image: 'https://resource.kingsome.cn/dalmatian/wapp/share.jpg'
      }
    }
  }
  $isIos() {
    let info = wepy.getSystemInfoSync();
    console.log(info);
    return (info.platform && (/^ios/.test(info.platform.toLowerCase()) || /^devtools/.test(info.platform.toLowerCase())));
  }
  // 判断是否是iPhone X系列
  $isIpx() {
    const info = wepy.getSystemInfoSync();
    return (info.model && /^iphone x/.test(info.model.toLowerCase()))
  }
  jcLog(t, e, n) {
    try {
      n = n || 'debug';
      e = e || n.toUpperCase();
      const i = 'error' === n ? 'background: #c4161e; color: #fff' : 'warning' === n ? 'background: #ff8c1c; color: #fff' : 'info' === n ? 'background: #ff0080; color: #fff' : 'background: #44a5ab; color: #fff',
        r = console.log('%c %c %c ==JCLOGGER== %c %c %c ' + e + ' ', 'background: #9854d8', 'background: #6c2ca7', 'color: #fff; background: #450f78;', 'background: #6c2ca7', 'background: #9854d8', i, void 0 !== t ? t : '');
      return console.log.bind(console, r);
    } catch (e) {
      return console.log(e);
    }
  }
  /**
   * 根据class获取宽度
   * */
  queryViewWidth(viewClass) {
    //创建节点选择器
    return new Promise(function(resolve) {
      let query = wx.createSelectorQuery();
      query.select('.' + viewClass).boundingClientRect(function(rect) {
        resolve(rect.width);
      }).exec();
    });
  }
  /**
   * 获取本地存储
   * @param {string} key
   * */
  $getStorage(key) {
    return wepy.getStorageSync(key);
  }
  /**
   * 保存数据至本地存储
   * @param {string} key
   * @param {object} value
   * */
  $setStorage(key, value) {
    wepy.setStorageSync(key, value);
  }
}
