import wepy from 'wepy'
import http from '../utils/http';
import jcEvent from '../common/jc-event';
import storage from '../utils/storage';

export default class apiMixin extends wepy.mixin {
  async $clienLogin() {
    let account = wepy.getStorageSync('account');
    let data = {
      account_id: account.account_id,
      session_id: account.session_id,
      open_id: account.openid,
      unionid: account.unionid,
    };
    let inviterId = wepy.getStorageSync('temp_inviter_id');
    wepy.removeStorageSync('temp_inviter_id');
    if (inviterId) {
      data.share_uid = inviterId;
    }
    if (global.dataType === 'game') {
      let wxData = await wepy.login();
      console.log('wx.login: ', wxData);
      data.code = wxData.code;
    }
    let res = await http.post('/api/emulated/user_login', data);
    wepy.setStorageSync('mp_token', res.token);
  }
  $isClientLogin() {
    return !!wepy.getStorageSync('mp_token');
  }
  /**
   * 观看广告成功上报
   * @param {String} aid
   * */
  async $adSuccess(aid) {
    if (aid) {
      try {
        await http.post('/api/emulated/update_ad', {aid: aid});
        jcEvent.emit(jcEvent.events.NEED_UPDATE_SCORE, {});
      } catch (err) {
        console.log('update user score error');
      }
    }
  }
  /**
   * 观看广告前请求广告记录id
   * @param {String} text 跳转页面上主标题
   * @param {String} context
   * */
  async $requestAd(text, context) {
    let self = context;
    let res;
    try {
      res = await http.post('/api/emulated/request_ad', {});
    } catch (err) {

    }
    if (res && res.errcode === 0) {
      // 用户触发广告后，显示激励视频广告
      self.currentAid = res.aid;
      if (self.videoAd) {
        try {
          await self.videoAd.show();
        } catch (err) {
          try {
            await self.videoAd.load();
            await self.videoAd.show();
          } catch (err) {
            console.log('激励视频 广告显示失败')
            // self.$toJumpPage(res.aid, text);
          }
        }
      } else {
        // self.$toJumpPage(res.aid, text);
        console.log('获取激励视频失败');
      }
    } else {
      console.log('request ad with errmsg: ' + res.errmsg);
    }
  }

  async $registVideoAd(context) {
    let self = context;
    if (wx.createRewardedVideoAd && global.dataType === 'game') {
      self.videoAd = wx.createRewardedVideoAd({
        adUnitId: 'adunit-4c4e7a71626fedc5'
      })
      self.videoAd.onClose(async (status) => {
        if (status && status.isEnded || status === undefined) {
          console.log('激励广告播放结束');
          await self.$adSuccess(self.currentAid);
        } else {
          console.log('激励广告播放失败');
        }
      });
      self.videoAd.onError(() => {
        console.log('激励广告载入失败');
        // self.$toJumpPage(self.currentAid, '');
        wepy.showToast({ title: '获取视频失败' });
      });
    }
  }
  /**
   * 获取指定页面的热更新代码
   * @param {string} pageName 页面名称
   * @param {boolean} needUpdate 如果本地没有代码, 是否从服务器更新
   * @return {string} code 当前页面上的热更新代码
   * */
  async $getHotCode(pageName, needUpdate = false) {
    const cfgObj = wepy.getStorageSync('cfg');
    const codeVersion = cfgObj.code_version;
    const localCode = wepy.getStorageSync('code');
    return new Promise(async (resolve, reject) => {
      if (!codeVersion) {
        return reject('');
      }
      if (localCode && localCode.codeVersion === codeVersion && localCode.clientVersion === global.version) {
        return resolve(localCode.code[pageName]);
      }
      if (needUpdate) {
        try {
          let res = await http.post('/api/common/check_update', {codeVersion: codeVersion, clientVersion: global.version});
          let obj = {
            codeVersion: codeVersion,
            clientVersion: global.version,
            code: res.code,
          }
          wepy.setStorageSync('code', obj);
          resolve(res.code[pageName]);
        } catch (err) {
          reject(err)
        }
      } else {
        resolve('');
      }
    })
  }
  /**
   * POST网络请求, 集成cache
   * @param {string} url  数据请求地址
   * @param {object} postData post的数据
   * @param {string} key  本地存储的key
   * @param {boolean} refresh 是否强制刷新缓存
   * */
  async $postWithCache(url, postData, key, refresh = false) {
    return new Promise(async (resolve, reject) => {
      let res;
      if (refresh) {
        try {
          res = await http.post(url, postData);
        } catch (err) {
          reject(err);
        }
        storage.put(key, res);
        resolve(res)
      } else {
        let data = storage.get(key);
        if (data) {
          resolve(data);
        } else {
          try {
            res = await http.post(url, postData);
          } catch (err) {
            reject(err);
          }
          storage.put(key, res);
          resolve(res)
        }
      }
    })
  }
}
